###############################################################################
#Definition of a preliminary descendant class of M.Kohl's class ParamFamily
#will be the new definition of his ParamFamily in next release
###############################################################################

require("ROptEst")
setClass("nParamFamily", representation(modifparameter = "function"), 
            prototype(name = "parametric family of probability measures",
                      distribution = new("Norm"),
                      distrSymm = new("NoSymmetry"),
                      props = character(0),
                      modifparameter = function(theta) Norm(mean=theta),
                      param = new("ParamFamParameter", main = 0, trafo = as.matrix(1))),
            contains = "ParamFamily")

nParamFamily <- function(name, distribution = Norm(), distrSymm, 
                         modifparameter, main = 0, nuisance, trafo, param, 
                        props = character(0)){
    if(missing(name)) 
        name <- "parametric family of probability measures"
    if(missing(distrSymm)) distrSymm <- NoSymmetry()
    if(missing(param)) 
        param <- ParamFamParameter(name = paste("parameter of", name), 
                        main = main, nuisance = nuisance, trafo = trafo)
    if(missing(modifparameter)) 
       stop(cat(paste("Please enter a function(theta) with value a new instance of", 
                      "slot distribution with corresponding parameter value theta.",
                      "example (normal location) function(theta) Norm(mean=theta)\n", collapse="",sep="")))
    return(new("nParamFamily", name = name, distribution = distribution, 
               distrSymm = distrSymm, param = param, props = props,
               modifparameter = modifparameter))
}


## access methods
if(!isGeneric("modifparameter")){ 
    setGeneric("modifparameter", function(object) standardGeneric("modifparameter"))
}

setMethod("modifparameter", "nParamFamily", function(object) object@modifparameter)
setMethod("dimension", "ParamFamParameter", function(object) length(object@main))


############################################################################################
#
#distances for distributions <-> (empirical) data
#
############################################################################################

setMethod("KolmogorovDist", signature(e1 = "UnivariateDistribution",
                                      e2 = "numeric"),
    function(e1, e2){
        ###code from function ks.test:
        x <- e2[!is.na(e2)]
        n <- length(x)
        x <- p(e1)(sort(x)) - (0:(n - 1))/n
        ###
        res <- max(c(x, 1/n - x))
        names(res) <- "Kolmogorov distance"
        return(res)
    })



setMethod("HellingerDist", signature(e1 = "DiscreteDistribution",
                                     e2 = "numeric"),
    function(e1, e2){
        d2 <- table(e2)/length(e2)
        e21 <- setdiff(support(e1),sort(unique(e2)))
        d10 <- d(e1)(sort(unique(e2)))
        d11 <- d(e1)(e21)
        res <- sqrt(1/2)*sqrt(sum((sqrt(d2)-sqrt(d10))^2)+sum(d11))
        names(res) <- "Hellinger distance"
        return(res)
    })



negLoglikelihood <- function(e1, e2){
        res <- -sum(log(e1@d(e2)))
        names(res) <- "Negative Log-Likelihood"
        return(res)
    }


fitParam <- function(model, data0, criterium, initval0, searchinterval0,
                        ...) # ...  for control for optimization
   {ftoOptimize <- function(theta)
        {criterium(modifparameter(model)(theta),data0)}
#    if(!is(param(model), DiscreteParameter))
       {if(dimension(param(model)) == 1)
           {if(missing(searchinterval0)) searchinterval0 <- c(-Inf,Inf)  
            theta <- optimize(f = ftoOptimize, interval = searchinterval0, ...)$minimum
            names(theta) <- name(param(model))}
        else 
           {if(missing(initval0)) initval0 <- main(param(model))
            theta <- optim(par = initval0, f = ftoOptimize, ...)$par
            names(theta) <- names(main(param(model)))}
       }
   return(theta) 
   }
   
MLE <- function(ParaModel, data, initval, 
                searchinterval, ...)
       fitParam(model = ParaModel, data0 = data, criterium = negLoglikelihood, 
                        initval0 = initval, searchinterval0 = searchinterval, ...) 

MDE <- function(ParaModel, data, initval, distance = KolmogorovDist,
                searchinterval, ...)
       fitParam(model = ParaModel, data0 = data, criterium = distance, 
                        initval0 = initval, searchinterval0 = searchinterval, ...) 
